@@ -315,12 +315,8 @@ ldap_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, LDAP **ldap, int
 static gboolean
 gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 {
-	if (!bl->priv->ldap) {
-		return FALSE;
-	}
-
 	/* we need to reconnect if we were previously connected */
-	if (bl->priv->connected && ldap_status == LDAP_SERVER_DOWN) {
+	if ((bl->priv->connected && ldap_status == LDAP_SERVER_DOWN) || (!bl->priv->ldap && !bl->priv->connected)) {
 		g_mutex_lock (bl->priv->ldap_lock);
 		if (book_view)
 			book_view_notify_status (book_view, _("Reconnecting to LDAP server..."));
@@ -331,6 +327,7 @@ gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 			book_view_notify_status (book_view, "");
 
 		if (bl->priv->ldap != NULL) {
+			bl->priv->connected = TRUE;
 			g_mutex_unlock (bl->priv->ldap_lock);
 			return TRUE;
 		} else {
@@ -339,6 +336,7 @@ gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 		}
 	}
 	else {
+		printf("Connected and ldap is null sigh\n");
 		return FALSE;
 	}
 }
@@ -1646,11 +1644,13 @@ ldap_search_dtor (LDAPOp *op)
 	d(printf ("ldap_search_dtor (%p)\n", search_op->view));
 
 	/* unhook us from our EDataBookView */
+	printf ("ldap_search_dtor: Setting null inplace of %p in view %p\n", op, search_op->view);
 	g_object_set_data (G_OBJECT (search_op->view), "EBookBackendGAL.BookView::search_op", NULL);
 
 	bonobo_object_unref (search_op->view);
 
-	g_free (search_op);
+	if (!search_op->aborted)
+		g_free (search_op);
 }
 
 #if ENABLE_CACHE		
@@ -1764,10 +1764,12 @@ start_book_view (EBookBackend  *backend,
 #endif			
 			printf("Not marked for offline or cache not there\n");
 			if (!bl->priv->ldap) {
-				printf("%s(%d):%s: no ldap :(\n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION);
-				e_data_book_view_notify_complete (view,
-								  GNOME_Evolution_Addressbook_InvalidQuery);
-				return;
+				if (!gal_reconnect (bl, view, 0)) {
+					printf("%s(%d):%s: no ldap :(\n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION);
+					e_data_book_view_notify_complete (view,
+									  GNOME_Evolution_Addressbook_InvalidQuery);
+					return;
+				}
 			}
 
 			/* we start at 1 so the user sees stuff as it appears and we
@@ -1793,21 +1795,24 @@ start_book_view (EBookBackend  *backend,
 			}
 
 			do {
-				book_view_notify_status (view, _("Searching..."));
-
-				g_mutex_lock (bl->priv->ldap_lock);
-				printf("%s(%d):%s: starting \n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION);
-				ldap_err = ldap_search_ext (bl->priv->ldap, LDAP_ROOT_DSE,
-							    LDAP_SCOPE_SUBTREE,
-							    ldap_query,
-							    search_attrs, 0,
-							    NULL, /* XXX */
-							    NULL, /* XXX */
-							    NULL, /* XXX timeout */
-							    view_limit,
-							    &search_msgid);
-				g_mutex_unlock (bl->priv->ldap_lock);
-				printf("%s(%d):%s: %d\n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION, ldap_err);
+				if (bl->priv->ldap) {
+					book_view_notify_status (view, _("Searching..."));
+	
+					g_mutex_lock (bl->priv->ldap_lock);
+					printf("%s(%d):%s: starting \n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION);
+					ldap_err = ldap_search_ext (bl->priv->ldap, LDAP_ROOT_DSE,
+								    LDAP_SCOPE_SUBTREE,
+								    ldap_query,
+								    search_attrs, 0,
+								    NULL, /* XXX */
+								    NULL, /* XXX */
+								    NULL, /* XXX timeout */
+								    view_limit,
+								    &search_msgid);
+					g_mutex_unlock (bl->priv->ldap_lock);
+					printf("%s(%d):%s: %d\n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION, ldap_err);
+				} else 
+					bl->priv->connected = FALSE;
 			} while (gal_reconnect (bl, view, ldap_err));
 
 			g_free (ldap_query);
@@ -1825,17 +1830,18 @@ start_book_view (EBookBackend  *backend,
 			}
 			else {
 				LDAPSearchOp *op = g_new0 (LDAPSearchOp, 1);
-
+				
 				d(printf ("adding search_op (%p, %d)\n", view, search_msgid));
 				printf("%s(%d):%s: adding search \n", __FILE__, __LINE__, G_GNUC_PRETTY_FUNCTION);
 				op->view = view;
+				op->aborted = FALSE;
 
 				bonobo_object_ref (view);
 
 				ldap_op_add ((LDAPOp*)op, E_BOOK_BACKEND (bl), NULL, view,
 					     0, search_msgid,
 					     ldap_search_handler, ldap_search_dtor);
-
+				printf("start_book_view: Setting op %p in book %p\n", op, view);
 				g_object_set_data (G_OBJECT (view), "EBookBackendGAL.BookView::search_op", op);
 			}
 #if ENABLE_CACHE			
@@ -1853,8 +1859,12 @@ stop_book_view (EBookBackend  *backend,
 	d(printf ("stop_book_view (%p)\n", view));
 
 	op = g_object_get_data (G_OBJECT (view), "EBookBackendGAL.BookView::search_op");
-	if (op)
+	printf("STOP BOOK VIEW: Getting op %p from view %p\n", op, view);
+	if (op) {
+		op->aborted = TRUE;
 		ldap_op_finished ((LDAPOp*)op);
+		g_free (op);
+	}
 }
 
 static void
@@ -2021,9 +2031,14 @@ static int dosearch(
 	LDAPMessage		*res, *msg;
 	ber_int_t		msgid;
 	static int count = 0;
-
+	char *ssize = getenv("LDAP_LIMIT");
+	int size = 0;
+	
+	if (ssize && *ssize) 
+		size = atoi(ssize);
+	
 	rc = ldap_search_ext( ld, base, scope, value, attrs, attrsonly,
-		sctrls, cctrls, timeout, -1 /*LDAP_NO_LIMIT*/, &msgid );
+		sctrls, cctrls, timeout, size /*LDAP_NO_LIMIT*/, &msgid );
 
 	if( rc != LDAP_SUCCESS ) {
 		return( rc );
@@ -2465,7 +2480,7 @@ load_source (EBookBackend *backend,
  				return GNOME_Evolution_Addressbook_OtherError;
  			}
  
- 			db_error = env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
+ 			db_error = (*env->open) (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
  			if (db_error != 0) {
  				env->close (env, 0);
  				g_warning ("db_env_open failed with %d", db_error);
@@ -2490,7 +2505,7 @@ load_source (EBookBackend *backend,
  			return GNOME_Evolution_Addressbook_OtherError;
  		}
  
- 		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
+ 		db_error = (*db->open) (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
  
  		if (db_error == DB_OLD_VERSION) {
  			db_error = e_db3_utils_upgrade_format (filename);
@@ -2502,7 +2517,7 @@ load_source (EBookBackend *backend,
  				return GNOME_Evolution_Addressbook_OtherError;
  			}
  
- 			db_error = db->open (db, NULL,filename, NULL, DB_HASH, DB_THREAD, 0666);
+ 			db_error = (*db->open) (db, NULL,filename, NULL, DB_HASH, DB_THREAD, 0666);
  		}
  
  		bl->priv->file_db = db;
@@ -2521,7 +2536,7 @@ load_source (EBookBackend *backend,
  					return GNOME_Evolution_Addressbook_OtherError;
  			}
  
- 			db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_CREATE | DB_THREAD, 0666);
+ 			db_error = (*db->open) (db, NULL, filename, NULL, DB_HASH, DB_CREATE | DB_THREAD, 0666);
  			if (db_error != 0) {
  				g_warning ("db->open (...DB_CREATE...) failed with %d", db_error);
  			}
