@@ -21,9 +21,15 @@ guint32  mail_util_props_to_camel_flags (E2kProperties *props,
 char *   mail_util_extract_transport_headers (E2kProperties *props);
 
 gboolean mail_util_demangle_delegated_meeting (GString *body,
-					       const char *delegator_cn,
-					       const char *delegator_email,
-					       const char *delegator_cal_uri);
+					    const char *delegator_cn,
+					    const char *delegator_email,
+					    const char *delegator_cal_uri,
+					    const char *delegatee_email);
+
+gboolean
+mail_util_demangle_sender_field (GString *body,
+				const char *delegator_email,
+				const char *sender_email);
 
 #ifdef __cplusplus
 }
